#!/usr/bin/env python
# 
# CTL IDL compiler, based on idlParse.py ((c) 2003, Paul McGuire)
#
# (c) 2005 Boris Buegling
#

import sys
from pyparsing import *
from re import *
from string import *

# Punctuation
colon  	= Literal(":")
lbrace 	= Literal("{")
rbrace 	= Literal("}")
lbrack 	= Literal("[")
rbrack 	= Literal("]")
lparen 	= Literal("(")
rparen 	= Literal(")")
equals 	= Literal("=")
comma  	= Literal(",")
dot    	= Literal(".")
slash  	= Literal("/")
bslash 	= Literal("\\")
star   	= Literal("*")
semi   	= Literal(";")
langle 	= Literal("<")
rangle 	= Literal(">")
amp		= Literal("&").suppress()
plus	= Literal("+")
minus 	= Literal("-")
percent = Literal("%")
power 	= Literal("^")
pipe	= Literal("|")
tilde	= Literal("~")
not_	= Literal("!")

bracket = Literal("()")
pluseq	= Literal("+=")
minuseq	= Literal("-=")
stareq	= Literal("*=")
slasheq	= Literal("/=")
perceq	= Literal("%=")
poweq	= Literal("^=")
ampeq	= Literal("&=")
pipeq	= Literal("|=")
shl		= Literal("<<")
shr		= Literal(">>")
shleq	= Literal("<<=")
shreq	= Literal(">>=")
eqeq	= Literal("==")
noteq	= Literal("!=")
langleq = Literal("<=")
rangleq = Literal(">=")
and_	= Literal("&&")
or_		= Literal("||")
inc		= Literal("++")
dec		= Literal("--")
moo		= Literal("->*")
moo2	= Literal("[]")
new		= Literal("new")
new2	= Literal("new[]")
del_	= Literal("delete")
del2	= Literal("delete[]")
# :: . .* ?: sizeof typeid
  
# Keywords
const_		= Literal("const")
namespace_	= Literal("namespace")
op_			= Literal("operator")
typedef_	= Literal("typedef")
using_		= Literal("using")

# Types
bool_		= Literal("bool")
char_		= Literal("char")
void_		= Literal("void")

any_		= Literal("utl::any") | Literal("any")
vector_		= Literal("utl::vector")
array_		= Literal("array")
loc_		= Literal("ctl::location") | Literal("location")
str_		= Literal("std::string") | Literal("string")
	
int2_		= Literal("utl::int2")
int4_		= Literal("utl::int4")
int8_		= Literal("utl::int8")
real4_		= Literal("utl::real4")
real8_		= Literal("utl::real8")
	
uchar_		= Literal("utl::uchar")
uint2_		= Literal("utl::uint2")
uint4_		= Literal("utl::uint4")
uint8_		= Literal("utl::uint8")

# CPP keywords
define_		= (Literal("#") + Literal("define")).suppress()
endif_		= (Literal("#") + Literal("endif")).suppress()
ifndef_		= (Literal("#") + Literal("ifndef")).suppress()
include_	= (Literal("#") + Literal("include")).suppress()
struct_		= Literal("struct").suppress()
clas_		= Literal("class").suppress()

# CTL keywords
constr_		= Literal("CTL_Constructor")
method_		= Literal("CTL_Method")
smethod_	= Literal("CTL_StaticMethod")
func_		= Literal("CTL_Function")
functmpl_	= Literal("CTL_FunctionTmpl")
clbegin_	= Literal("CTL_ClassBegin").suppress()
clend_		= Literal("CTL_ClassEnd").suppress()
class_		= Literal("CTL_Class").suppress()
classtmpl_	= Literal("CTL_ClassTmpl").suppress()
libbegin_	= Literal("CTL_LibBegin").suppress()
libend_		= Literal("CTL_LibEnd").suppress()
lib_		= Literal("CTL_Library").suppress()
throws_		= Literal("Throws")


# Inline C code
typedef = typedef_ + restOfLine

# Grammar
identifier = Word(alphas, alphanums + "_").setResultsName("identifier")
keyword = (any_ | bool_ | char_ | int2_ | int4_ | int8_ | real4_ | real8_ | \
	str_ | uchar_ | uint2_ | uint4_ | uint8_ | void_ | loc_ | \
	array_ | vector_).setResultsName("keyword")
ident = keyword | identifier
	
numargs = Word(nums).setResultsName("numargs")
id = Word(nums).setResultsName("id")

op = (op_ + (bracket | star | plus | minus | slash | percent | power | \
	amp | pipe | tilde | not_ | equals | langle | rangle | pluseq | \
	minuseq | stareq | slasheq | perceq | poweq | ampeq | pipeq | \
	shl | shr | shleq | shreq | eqeq | noteq | langleq | rangleq | \
	and_ | or_ | inc | dec | moo | comma | moo2 | new | new2 | del_ | \
	del2)).setResultsName("op")
funcname = (op | ident).setResultsName("funcname")
	
type_ = Forward()
type1 = Optional(const_) + ident + langle + type_ + rangle
type2 = Optional(const_) + ident + Optional(star | amp)
type3 = lparen + Optional(const_) + ident + comma + lparen + type_ + \
	ZeroOrMore(comma + type_) + rparen + comma.suppress() + numargs + \
	rparen
type_ << (type1 | type2 | type3).setResultsName("type")

typelist1 = (lparen + rparen).suppress() + Optional(const_) + \
	(comma + "0").suppress()
typelist2 = lparen.suppress() + type_ + ZeroOrMore(comma + type_) + \
	rparen.suppress() + Optional(const_).suppress() + \
	(comma + numargs).suppress()
typelist = (typelist1 | typelist2).setResultsName("typelist")

funcsign = (type_ + comma + funcname + comma + \
	typelist).setResultsName("funcsign")
except_ = (id.suppress() + throws_.suppress() + \
	typelist).setResultsName('except')

constructor = (define_ + constr_ + id + typelist + \
	Optional(define_ + constr_.suppress() + \
	except_)).setResultsName("constructor")
method = (define_ + method_ + id + funcsign + \
	Optional(define_ + method_.suppress() + \
	except_)).setResultsName("method")
staticmethod = (define_ + smethod_ + id + funcsign + \
	Optional(define_ + smethod_.suppress() + \
	except_)).setResultsName("staticmethod")
function = (define_ + func_ + id + funcsign + \
	Optional(define_ + func_.suppress() + \
	except_)).setResultsName("function")
functiontmpl = (define_ + functmpl_ + id + funcsign + typelist + \
	comma + typelist + Optional(define_ + functmpl_.suppress() + \
	except_)).setResultsName("functiontmpl")
	
classentry = (method | staticmethod | constructor).setResultsName("classentry")
classbody = (include_ + clbegin_ + OneOrMore(classentry) + include_ + \
	clend_).setResultsName("classbody")
classname = (define_ + class_ + ident).setResultsName("classname")
_class = (classname + classbody).setResultsName("class")
# TODO: Handle template classes
classtmpl = (define_ + classtmpl_ + ident + comma + \
	typelist.setResultsName('type_param') + \
	classbody).setResultsName("classtmpl")

libraryentry = (_class | classtmpl | function | \
	functiontmpl).setResultsName("libraryentry")
librarybody = (include_ + libbegin_ + OneOrMore(libraryentry) + \
	include_ + libend_).setResultsName("librarybody")
libname = (define_ + lib_ + ident).setResultsName("libname")
library = (libname + librarybody).setResultsName("library")

bnf = Optional(include_ + Literal("<ctl.h>")).suppress() + \
	(library | _class)

# Ignore inline C code and comments for parsing
namespace = using_ + namespace_ + restOfLine
defs = (ifndef_ + restOfLine + define_ + restOfLine) | (endif_ + restOfLine)
singleLineComment = "//" + restOfLine
struct = struct_ + restOfLine
inlcode = Forward()
inlcode << lbrace + ZeroOrMore(inlcode | CharsNotIn("{}")) + rbrace + \
		Optional(semi)
clas = clas_ + identifier + inlcode

bnf.ignore(typedef)
bnf.ignore(struct)
bnf.ignore(clas)
bnf.ignore(inlcode)
bnf.ignore(namespace)
bnf.ignore(defs)
bnf.ignore(singleLineComment)
bnf.ignore(cStyleComment)
###############

first = True
def p_class (s, loks, toks):
	global outp
	cname = toks[0]
	if cname[-2:] == "RI":
		cname = cname[:-2]
	if 'type_param' in toks.keys():
		# TODO: Handle complex type parameters
		cname += '<' + toks['type_param'][0]
		for t in toks['type_param'][1:]:
			cname += ', ' + t
		cname += '>'
	outp.append('public class %s\n{' % cname)

def p_class2 (s, loks, toks):
	global outp
	outp.append('}')

def parse_exceptions (moo):
	ret = 'throws '
	for type in moo:
		if type in ('std::string', 'string'):
			ret += 'CTL.Types.CTLException '
	return ret[:-1]

def type2java (typ):
	if typ == 'const':
		return '@CTL.Annotate.const_'
	
	if typ in ('any', 'utl::any'):
		return 'CTL.CCompat.AnyObj'
	if typ in ('array', 'utl::vector', 'vector'):
		return 'CTL.CCompat.CArray'
	
	if typ in ('utl::int4', 'int4', 'uint4'):
		return 'Integer'
	if typ in ('utl::int8', 'int8', 'uint8'):
		return 'Long'
	if typ in ('utl::real4', 'real4'):
		return 'Float'
	if typ in ('utl::real8', 'real8'):
		return 'Double'
	if typ in ('bool'):
		return 'Boolean'
	
	if typ in ('string', 'std::string'):
		return unscope(typ).capitalize()
	
	if typ in ('location', 'ctl::location', 'tupel', 'reference'):
		return 'CTL.Types.'+unscope(typ).capitalize()
	
	return typ

def unscope (typ):
	if match('.*::.*', typ):
		typ = typ.split('::')[1]
	return typ

def p_func (s, loks, toks):
	global outp
	typ = toks[0]
	#print typ

	if (typ == str(method_)[1:-1]) or (typ == str(func_)[1:-1]):
		fname = toks['funcname'][0]
		# TODO: Some kind of support for 'operator'
		if fname.startswith('operator'):
			return
		id = toks['id']
		if 'except' in toks.keys():
			exceptions = parse_exceptions(toks['except'])
		else:
			exceptions = ''
		retval = list()
		for tok in toks['funcsign']:
			if tok == ',':
				break
			retval.append(type2java(tok))
		ll = len(retval) + 3
		retval = join(retval)
		i = 0
		args = ''
		brak = 0
		for tok in toks['funcsign'][ll:]:
			if tok == '(':
				brak += 1
				args += '( '
				continue
			if tok == ')':
				brak -= 1
				args += ') '
				continue
			if (tok == ',') and (brak == 0):
				args += 'arg'+str(i) + ', '
				i += 1
			else:
				args += type2java(tok) + ' '
		args += 'arg'+str(i)
		# TODO: hack
		args = sub("cstring < char >", "String", args)
		args = resolve_brackets(args)
	else:
		# TODO: Handle constructors, etc.
		return

	outp.append('@ReflWrap.sFID(%s) public ' % id + retval + ' ' + \
		fname + ' (' + args + ') ' + exceptions + '\n{')
	if retval != 'void':
		outp.append('return null;')
	outp.append('}')

def resolve_brackets (expr):
	for m in findall('\\(.*?\\) arg', expr):
		expr = expr.replace(m, m[2:-6]+m[-4:])
	if '(' in expr:
		for m in findall(', \\(.*\\)..', expr):
			if str(m[-1]).isdigit():
				expr = expr.replace(m, '<'+m[4:-4]+'>')
			else:
				expr = expr.replace(m, ', '+m[4:-3] + m[-2:])
		return resolve_brackets(expr)
	return expr

def p_lib (s, loks, toks):	
	# TODO: Library
	libname = toks[0]
	
def ri2java (str):
	try:
		libname.setParseAction(p_lib)	
		classname.setParseAction(p_class)
		constructor.setParseAction(p_func)
		method.setParseAction(p_func)
		staticmethod.setParseAction(p_func)
		# TODO: Functions not fully supported
		function.setParseAction(p_func)
		functiontmpl.setParseAction(p_func)
		tokens = bnf.parseFile(str)
		
	except ParseException, err:
		print >>sys.stderr, err.line
		print >>sys.stderr, " "*(err.column-1) + "^"
		print >>sys.stderr, err
	
###############
if __name__ == '__main__':
    if len(sys.argv) == 1:
		print "Usage: %s [CI]" % sys.argv[0]
    else:
		pkg = 'Impl'
		args = list()
		for arg in sys.argv[1:-1]:
			if arg.startswith('-P'):
				pkg = arg[2:]
			if arg.startswith('-D'):
				args.append(arg[2:])
				
		outp = list()
		outp.append('package %s;\n' % pkg)
		ri2java(sys.argv[-1])
		# TODO: hack
		outp.append('}')
		
		outp = '\n'.join(outp)
		for arg in args:
			repl = arg.split('=')
			outp = sub(repl[0], repl[1], outp)
		
		print outp
