package CTL.Comm;

import CTL.*;
import CTL.Types.*;

import com.jcraft.jsch.*;
import java.io.*;

/** Wrapper for the jsch SSHv2 API */
public class SSHv2 implements Runnable
{
	/** Command to invoke */
	private String start;
	/** Location of the remote machine */
	private Location loc;
	
	/** Initialization
	 * @param loc Location
	 */
	public SSHv2 (Location loc, Communicator comm)
	{
		this.loc = loc;
		this.start = loc.path()+"/"+loc.cmd()+" tcp:"+comm.pid().host()+":"+
			comm.pid().port()+":"+Env.grp.nextID();
	}

	/** Runs the command specified by the Location on the remote machine */
	public void run ()
	{
		try
		{
			if ((loc==null)||(loc.path()==null)||(loc.cmd()==null))
			{
				Env.log.log_msg(Logger.ERR, "Location not initialized!");
				throw new RuntimeException("No location to connect to.");
			}
			
			JSch jsch = new JSch();
			jsch.setKnownHosts(System.getProperty("user.home")+
				"/.ssh/known_hosts");
		
			Env.log.log_msg(Logger.INFO, loc.toString());
			if (loc.user==null)
				loc.user=System.getProperty("user.name");
			Session sess = jsch.getSession(loc.user, loc.host(), 
				loc.port());
			loc.setPassword(sess);
			sess.connect();

			PipedOutputStream out = new PipedOutputStream();
			PipedInputStream in = new PipedInputStream(out);
			BufferedInputStream read = new BufferedInputStream(in);
			PipedOutputStream out2 = new PipedOutputStream();
			PipedInputStream in2 = new PipedInputStream(out2);
			BufferedInputStream read2 = new BufferedInputStream(in2);

			Channel chan = sess.openChannel("exec");
			if (chan==null)
			{
				Env.log.log_msg(Logger.ERR, "Session could not be opened.");
				throw new RuntimeException("SSH connection refused.");
			}

			((ChannelExec)chan).setCommand(start);
			chan.connect();
			if (chan.isEOF())
				throw new RuntimeException("SSH connection reset.");
			chan.setOutputStream(out);
			//chan.setInputStream(System.in);
			((ChannelExec)chan).setErrStream(out2);
			Env.log.log_msg(Logger.INFO, "Service spawned via SSH.");

			while (!chan.isEOF())
			{
				StringBuffer buf1 = new StringBuffer();
				StringBuffer buf2 = new StringBuffer();
			
				while (read.available()>0)
					buf1.append((char)read.read());
				while (read2.available()>0)
					buf2.append((char)read2.read());

				if (buf1.length()>0)
					Env.log.log_msg(Logger.INFO, buf1.toString());
				if (buf2.length()>0)
					Env.log.log_msg(Logger.ERR, buf2.toString());
			}
			sess.disconnect();	
		}
		catch (Exception e) 
		{
			Env.log.log_msg(Logger.ERR, e.toString());
			Env.log.log_msg(Logger.ERR, RUtil.stackTrace(e));
			throw new RuntimeException("Unexpected SSH connection error.");
		}
	}
}
