package CTL;

import CTL.Serialize.*;
import CTL.Types.*;
import ReflWrap.*;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

/** Handle reading and writing of Java standard library types.
 *	Types currently handled:
 *		* LinkedList
 *		* TypeTree (CTL specific type)	
 */
public class Java
{
	public static <T extends Object> void writeLinkedList (SerialOut out, 
		LinkedList<T> list) throws IOException, IllegalAccessException, 
		InvocationTargetException, CTLException
	{
		out.writeInt(list.size());
		for (T o : list)
			out.serialWrite(o);
	}

	public static <T extends Object> LinkedList<T> readLinkedList (SerialIn in, 
		TypeTree targs) throws IOException, IllegalAccessException,
		InvocationTargetException, ClassNotFoundException, 
		InstantiationException
	{
		LinkedList<T> list = new LinkedList<T>();
		int sz = in.readInt();
		for (int i=0;i<sz;i++)
		{
			T moo = (T)in.serialRead(targs);
			list.add(moo);
		}
		return list;
	}

	public static void writeTypeTree (SerialOut out, TypeTree tree) throws
		IOException, IllegalAccessException, InvocationTargetException, 
		CTLException
	{
		out.serialWrite(tree.klasse().getName());
		out.serialWrite(tree.targs());
	}

	public static TypeTree readTypeTree (SerialIn in) throws IOException, 
		IllegalAccessException, InvocationTargetException, 
		ClassNotFoundException, InstantiationException
	{
		Class klasse = Class.forName(in.readString());
		LinkedList<TypeTree> targs = (LinkedList<TypeTree>)
			in.serialRead(LinkedList.class, TypeTree.class);
		return new TypeTree(klasse, targs);
	}
}
