package CTL;

import java.io.*;
import java.util.*;

/** Filter-based logging class */
public class Logger
{
	/** Error */
	public static final int ERR = 2;
	/** Warning */
	public static final int WARN = 3;
	/** Information */
	public static final int INFO = 4;
	/** Debugging information, level 0 */
	public static final int DBG = 5;
	/** Debugging information, level 1 */
	public static final int DBG1 = 6;
	/** Debugging information, level 2 */
	public static final int DBG2 = 7;
	/** All information */
	public static final int ALL = Integer.MAX_VALUE;
	
	/** The file to write to */
	private FileWriter file;
	/** Filter level of this Logger (default: anything equally or more 
	 * important than warning) */
	private static int loglevel = WARN;

	/** Generate a new Logger with default filter
	 * @param fname Logfile
	 */
	public Logger (String fname)
	{
		try
		{
			file = new FileWriter(fname);
		}
		catch (IOException e)
		{
			System.err.println("Logging error.");
		}
	}

	/** Generate a new Logger with user-defined filter treshold
	 * @param fname Logfile
	 * @param loglevel Filter treshold
	 */
	public Logger (String fname, int loglevel)
	{
		this(fname);
		this.loglevel = loglevel;
	}

	/** Close the underlying file */
	public void close ()
	{
		try
		{
			file.close();
		}
		catch (IOException e)
		{
			System.err.println("Logging error.");
		}
	}

	/** Write a message to the logfile
	 * @param lvl Importance of this message, if equally or more important
	 * than the filter treshold, it will be logged
	 * @param msg Log message
	 */
	public void log_msg (int lvl, String msg)
	{
		try
		{
			if (lvl<=loglevel)
			{
				file.write(gettimeofday()+"  "+msg+"\n");
				file.flush();
			}
		}
		catch (IOException e)
		{
			System.err.println("Logging error.");
		}
	}

	/** Determine current time 
	 * @return Pretty-printed string containing the time of day
	 */
	private String gettimeofday ()
	{
		//String[] ids = TimeZone.getAvailableIDs(1 * 60 * 60 * 1000);
		SimpleTimeZone pdt = new SimpleTimeZone(1 * 60 * 60 * 1000, 
			"Europe/Berlin");
		Calendar cal = new GregorianCalendar(pdt);
		Date date = new Date();
		cal.setTime(date);

		StringBuffer buf = new StringBuffer();
		Formatter fmt = new Formatter(buf);
		fmt.format("%2d:%2d:%2d", cal.get(Calendar.HOUR_OF_DAY),
			cal.get(Calendar.MINUTE), cal.get(Calendar.SECOND));
		
		/*buf.append(cal.get(Calendar.HOUR_OF_DAY));
		buf.append(":"+cal.get(Calendar.MINUTE));
		buf.append(":"+cal.get(Calendar.SECOND));*/
		
		return buf.toString();
	}
}
