package CTL;

import CTL.Streams.*;
import CTL.Types.*;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

// TODO: Finish message queue implementation and use it
/** Queue for CTL messages */
public class MessageQueue
{
	/** ID-Header map */
	private HashMap<Integer, Header> map0;
	/** ID-Message map */
	private HashMap<Integer, byte[]> map1;	
	
	/** Generate a new queue */
	public MessageQueue ()
	{
		map0 = new HashMap<Integer, Header>();
		map1 = new HashMap<Integer, byte[]>();
	}

	/** Read a message from the stream and enqueue it
	 * @param in Input stream
	 */
	public void readFrom (OIStream in) throws IOException, ClassNotFoundException,
		InstantiationException, IllegalAccessException, InvocationTargetException
	{
		Header head = in.serialRead(Header.class);
		
		byte[] payload = new byte[(int)head.getSize()];
		int r = in.read(payload);
		if (r<payload.length)
			Env.log.log_msg(Logger.ERR, "Requested "+payload.length+" byte"+
				", but only "+r+" byte could be read.");
		
		enqueue(head, payload);
	}

	/** Enqueue a message
	 * @param head Header
	 * @param _msg Message
	 */
	public void enqueue (Header head, byte[] _msg)
	{
		int len = _msg.length;
		
		if ((_msg==null) || (len<5))
			return;

		int tag = _msg[len-1] + (_msg[len-2] << 8) + (_msg[len-3] << 16)
			+ (_msg[len-4] << 24);
		byte[] msg = new byte[len-4];
		for (int i=0;i<(len-4);i++)
			msg[i] = _msg[i];
		enqueue(tag, head, msg);
	}

	/** Enqueue a message with an user-defined ID
	 * @param tag ID
	 * @param head Header
	 * @param msg Message
	 */
	public void enqueue (int tag, Header head, byte[] msg)
	{
		map0.put(tag, head);
		map1.put(tag, msg);
	}

	/** Retrieve a message
	 * @param tag ID
	 * @return Message or null if not found
	 */
	public byte[] payload (int tag)
	{
		byte[] ret = map1.get(tag);
		map1.remove(tag);
		return ret;
	}

	/** Retrieve the header of a message
	 * @param tag ID
	 * @return Message header or null if not found
	 */
	public Header header (int tag)
	{
		Header head = map0.get(tag);
		map0.remove(tag);
		return head;
	}

	/** Check if a message with a specific ID is enqueued 
	 * @param tag ID
	 * @return True if present, false otherwise
	 */
	public boolean contains (int tag)
	{
		return map0.containsKey(tag) && map1.containsKey(tag);
	}

	/** Just a small testcase */
	public static void main (String[] args)
	{
		MessageQueue msg = new MessageQueue();
		byte[] moo = new byte[5];
		
		moo[0] = 23;
		moo[1] = (byte)(555 >> 24);
		moo[2] = (byte)(555 >> 16);
		moo[3] = (byte)(555 >> 8);
		moo[4] = (byte)555;
		
		msg.enqueue(null, moo);
		if (msg.contains(555))
			if (msg.payload(555)[0]!=23)
				System.exit(1);
	}
}
