package CTL;

import CTL.Comm.*;
import CTL.Streams.*;
import CTL.Types.*;

import java.io.*;
import java.net.*;

/** Communication process */
public class Process
{
	/** Location of the remote communication partner */
	protected Location loc;

	private static Timer t = new Timer();
	private boolean daemon = false;
	private int remote_id = -1;

	public int ID ()
	{
		return remote_id;
	}

	/** Generate a new Process
	 * @param loc Location
	 */
	public Process (Location loc)
	{
		if (loc==null)
		{
			Env.log.log_msg(Logger.WARN, "Location is empty.");
			return;
		}
		
		this.loc = loc;

		try
		{
			if (loc.path() == null)
			{
				// TODO: Hack
				remote_id = 0;
				Env.grp = new Group(loc.host(), loc.port(), 1, 2, 
					loc.linkage(), false);
				daemon = true;
			}
			else
			{
				// TODO: Hack
				remote_id = 1;
				Env.grp = new Group(loc.host(), loc.port(), 0, 2, 
					loc.linkage());
				startService(loc, Env.comm[remote_id]);
				Env.comm[remote_id].accept();
				Env.grp.writeVersion(Env.comm[remote_id]);
				Env.grp.HS_recv();
				Env.log.log_msg(Logger.INFO, 
					"Service started in "+t.timestr(t.stop2())+" seconds");
			}

			Thread thr = new Thread(Env.grp);
			thr.start();
			while (!Env.grp.running());
		}
		catch (Exception e)
		{
			RUtil.except(e);
			throw new RuntimeException("Could not create link.");
		}
	}

	/** Start the remote communication service
	 * @param loc Location
	 */
	private static void startService (Location loc, Communicator comm)
	{
		SSHv2 cli = new SSHv2(loc, comm);
		Thread thr = new Thread(cli);
		thr.start();
	}

	/** Stop the remote communication service */
	public void stopService ()
	{
		stopService(ID(), daemon);
	}
	
	public static void stopService (int you, boolean dmn)
	{
		try
		{
			if (dmn)
			{
				BuffyOut out = new BuffyOut();
				Header head = new Header(0, Remote.EOC, Env.comm[you]);
				out.o.serialWrite(head);
				out.o.flush();
				Env.comm[you].send(out.getBytes());
			}
			else
			{
				Header head = new Header(0, Remote.RMI, Env.comm[you]);
				FID fid = new FID((short)3, "");
				IStream2 args = new IStream2();
				args.write(Env.grp.grp[Env.grp.myID()].pid());
				args.write("EOF.");
				Env.grp.terminate();
				Remote.call(Env.comm[you], head, Env.grp.grp[you].id(),
					fid, args, new rPointer(0, Env.comm[you]));
				
			}
			
			Env.comm[you].close();
		} catch (Exception e) {}

		t.stop();
		System.out.println("Total execution time "+t+" seconds.");
		System.out.println("Clean termination.");
	}
}
