package CTL.Serialize;

import CTL.*;
import CTL.Streams.*;
import CTL.Types.*;
import ReflWrap.*;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

public abstract class SerialIn extends DataInputStream2
{
	public SerialIn (InputStream in) throws IOException
	{
		super(in);
	}
	
	public abstract String readString () throws IOException;
	public abstract String readWString () throws IOException;
	
	// TODO: Support for parametrized types in readArray and readTupel
	public abstract <T extends Object> T[] readArray (Class<T> type, 
		LinkedList<TypeTree> params) throws IOException, ClassNotFoundException, 
		InstantiationException, IllegalAccessException, 
		InvocationTargetException;
	public abstract <T extends Object> T[] __readArray (Class<T> type, 
		LinkedList<TypeTree> params, int len) throws IOException, 
		ClassNotFoundException, InstantiationException, IllegalAccessException, 
		InvocationTargetException;
	public abstract Tupel readTupel (Class[] types) throws IOException, 
		ClassNotFoundException, InstantiationException, IllegalAccessException,
		InvocationTargetException, CTLException;
	public abstract Object readObject () throws IOException, 
		ClassNotFoundException, InstantiationException, IllegalAccessException,
		InvocationTargetException;
	
	public Object serialRead (Class... args) throws IOException, 
		ClassNotFoundException, InstantiationException, 
		IllegalAccessException, InvocationTargetException
	{
		return serialRead(new TypeTree(args));
	}

	public <T extends Object> T serialRead (Class<T> klass) throws 
		IOException, ClassNotFoundException, InstantiationException,
		IllegalAccessException, InvocationTargetException
	{
		return (T)serialRead(new TypeTree(klass));
	}
			
	/** Read an object from the stream
	 * @param tree TypeTree to read
	 * @return Value
	 */
	public Object serialRead (TypeTree tree) throws IOException, 
		ClassNotFoundException, InstantiationException, 
		IllegalAccessException, InvocationTargetException
	{
		Class klass = tree.getType();
		TypeTree[] targs = tree.getTypeArray();
		ClassInfo cinfo = new ClassInfo(klass);

		Env.log.log_msg(Logger.DBG2, 
			"Reading: "+klass+" "+Arrays.toString(targs));

		try
		{
			if (klass.equals(String.class))
				return readString();
			
			if (klass.isArray())
			{
				LinkedList<TypeTree> list = new LinkedList<TypeTree>();
				for (int i=0;i<targs.length;i++)
					list.add(targs[i]);
				return readArray(cinfo.arrayType(), list);
			}

			if (klass.equals(Boolean.class) || klass.equals(Boolean.TYPE))
				return readBoolean();
			if (klass.equals(Byte.class) || klass.equals(Byte.TYPE))
				return readByte();
			if (klass.equals(Character.class) || klass.equals(Character.TYPE))
				return readChar();
			if (klass.equals(Double.class) || klass.equals(Double.TYPE))
				return readDouble();
			if (klass.equals(Float.class) || klass.equals(Float.TYPE))
				return readFloat();
			if (klass.equals(Integer.class) || klass.equals(Integer.TYPE))
				return readInt();
			if (klass.equals(Long.class) || klass.equals(Long.TYPE))
				return readLong();
			if (klass.equals(Short.class) || klass.equals(Short.TYPE))
				return readShort();
			
			// Handle Java standard library types
			if (klass.equals(LinkedList.class))
				return Java.readLinkedList(this, targs[0]);
			
			if (klass.equals(TypeTree.class))
				return Java.readTypeTree(this);
			
			if (!cinfo.implementing("CTL.Serialize.Writable"))
			{
				String msg = "OIStream: Class "+cinfo.fqcn()+
					" does not implement the Writable-Interface.";
				System.out.println(RUtil.stackTrace());
				Env.log.log_msg(Logger.ERR, msg);
				throw new RuntimeException(msg);
			}

			ConstructInfo constr = new ConstructInfo(cinfo.constructors()[0]);
			Object[] args = new Object[constr.params().length];
			for (int i=0;i<constr.params().length;i++)
			{
				args[i] = Refl.defaultVal2(constr.params()[i]);
				Env.log.log_msg(Logger.DBG2, constr.params()[i]+": "+args[i]);
			}
		
			Object ret = constr.c().newInstance(args);

			//System.out.println(klass+":::"+Arrays.toString(targs));
			
			if (cinfo.implementing("ReflWrap.TemplHack"))
				((TemplHack)ret).setTypes(targs);

			((Writable)ret).read(this);
			return ret;
		}
		catch (Exception e)
		{
			if (Env.daemon)
				return null;
			if ((Env.grp==null)||(Env.grp.running()))
			{
				Env.log.log_msg(Logger.ERR, "serialRead Exception"+
					" while reading "+klass+".");
				RUtil.except(e);
			}
		}
		
		return null;	
	}
}
