package CTL.Streams;

import CTL.*;
import CTL.Serialize.*;
import CTL.Types.*;
import ReflWrap.*;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

/** ObjectInputStream clone for the CTL */
public class OIStream extends SerialIn implements InStream
{
	private SerialIn stream;
	
	/** Generate a new stream
	 * @param in Underlying input stream
	 */
	public OIStream (InputStream in) throws IOException, CTLException
	{
		this(in, OOStream.Serial.CTL);
	}

	/** Constructor */
	public OIStream (InputStream in, OOStream.Serial type) throws IOException,
		CTLException
	{
		super(in);
		
		switch (type)
		{
			case CTL:
				stream = new CTLIn(in);
				break;
			default:
				throw new CTLException("Serializer does not exist.");
		}
	}
	
	public Object serialRead (TypeTree tree) throws IOException, 
		ClassNotFoundException, InstantiationException, 
		IllegalAccessException, InvocationTargetException
	{
		return stream.serialRead(tree);
	}
	
	public Object serialRead (Class... args) throws IOException, 
		ClassNotFoundException, InstantiationException, 
		IllegalAccessException, InvocationTargetException
	{
		return stream.serialRead(args);
	}

	public <T extends Object> T serialRead (Class<T> klass) throws 
		IOException, ClassNotFoundException, InstantiationException,
		IllegalAccessException, InvocationTargetException
	{
		return stream.serialRead(klass);
	}

	public int read (byte[] b) throws IOException
	{
		return stream.read(b);
	}

	public int readInt () throws IOException
	{
		return stream.readInt();
	}

	public String readString () throws IOException
	{
		return stream.readString();
	}

	public long readLong () throws IOException
	{
		return stream.readLong();
	}
	
	public <T extends Object> T[] readArray (Class<T> type, 
		LinkedList<TypeTree> params) throws IOException, ClassNotFoundException,
		InstantiationException, IllegalAccessException, 
		InvocationTargetException
	{
		return stream.readArray(type, params);
	}
		
	public <T extends Object> T[] __readArray (Class<T> type, 
		LinkedList<TypeTree> params, int len) throws IOException, 
		ClassNotFoundException, InstantiationException, IllegalAccessException, 
		InvocationTargetException
	{
		return stream.__readArray(type, params, len);
	}
 
	public Tupel readTupel (Class[] types) throws IOException, 
		ClassNotFoundException, InstantiationException, IllegalAccessException,
		InvocationTargetException, CTLException
	{
		return stream.readTupel(types);
	}
	
	public Object readObject () throws IOException, ClassNotFoundException,
		InstantiationException, IllegalAccessException,
		InvocationTargetException
	{
		return stream.readObject();
	}

	public boolean readBoolean () throws IOException
	{
		return stream.readBoolean();
	}

	public float readFloat () throws IOException
	{
		return stream.readFloat();
	}

	public String readWString () throws IOException
	{
		return stream.readWString();
	}

	public String readUTF () throws IOException
	{
		return stream.readUTF();
	}

	public String readLine () throws IOException
	{
		return stream.readLine();
	}

	public double readDouble () throws IOException
	{
		return stream.readDouble();
	}

	public char readChar () throws IOException
	{
		return stream.readChar();
	}

	public int readUnsignedShort () throws IOException
	{
		return stream.readUnsignedShort();
	}

	public int readUnsignedByte () throws IOException
	{
		return stream.readUnsignedByte();
	}

	public short readShort () throws IOException
	{
		return stream.readShort();
	}

	public byte readByte () throws IOException
	{
		return stream.readByte();
	}

	public int skipBytes (int n) throws IOException
	{
		return stream.skipBytes(n);
	}

	public void readFully (byte[] b, int off, int len) throws IOException
	{
		stream.readFully(b, off, len);
	}

	public void readFully (byte[] b) throws IOException
	{
		stream.readFully(b);
	}
}
