package CTL.Types;

import CTL.*;
import CTL.Serialize.*;
import CTL.Streams.*;

import java.io.*;
import java.util.*;

/** FunctionID */
public class FID extends Measure implements Writable
{
	/** Numerical ID */
	private short ID;
	/** Fully-qualified name */
	private String name;
	
	/** Generate a new FID
	 * @param ID Numerical ID
	 * @param name Fully-qualified name
	 */
	public FID (short ID, String name)
	{
		this.ID = ID;
		this.name = name;
	}

	/** Get a string representation of this object
	 * @return String
	 */
	public String toString ()
	{
		if (name.equals(""))
			return "ID: "+ID;
		
		return name+" - "+ID;
	}

	/** Get the numerical ID
	 * @return ID
	 */
	public short ID ()
	{
		return ID;
	}

	/** Get the fully-qualified name
	 * @return Name
	 */
	public String name ()
	{
		return name;
	}

	/** Extract the class name from the fully-qualified name of this function
	 * @return Class name
	 */
	public String _class ()
	{
		StringBuffer buf = new StringBuffer();
		StringTokenizer str = new StringTokenizer(name, ":");

		if (!str.hasMoreTokens())
			return "";

		while (str.hasMoreTokens())
		{
			String tok = str.nextToken();
			
			try
			{
				ID = (short)Integer.parseInt(tok.substring(0, tok.length()-1));
				break;
			} catch (NumberFormatException e) {}

			buf.append(tok+".");
		}
		
		return buf.toString().substring(0, buf.toString().length()-1);
	}

	/** Write this FunctionID to a stream
	 * @param out Output stream
	 */
	public void write (SerialOut out) throws IOException
	{
		out.writeShort(ID);
		out.writeString(name);
	}

	/** Read a FunctionID from a stream
	 * @param in Input stream
	 */
	public void read (SerialIn in) throws IOException, 
		ClassNotFoundException
	{
		ID = in.readShort();
		name = in.readString();
	}
}
