package CTL.Types;

import CTL.*;
import CTL.Comm.*;
import CTL.Serialize.*;
import CTL.Streams.*;

import java.io.*;
import java.lang.reflect.*;
import java.net.*;

/** CTL protocol header */
public class Header extends Measure implements Writable
{
	/** Payload size */
	private long size;
	/** Message tag */
	private int tag;
	/** Sender PeerID */
	private PeerID pid;
	/** Logical ID */
	private int logID;
	
	/** Default constructor */
	public Header (long size, int tag, PeerID pid, int logID)
	{
		this.size = size;
		this.tag = tag;
		this.pid = pid;
		this.logID = logID;
	}
	
	/** Constructor with host and port */
	public Header (long size, int tag, IPaddr host, int port, int logID)
	{
		this(size, tag, new PeerID(host, port), logID);
	}

	/** Constructor which gets the PeerID from a Communicator */
	public Header (long size, int tag, Communicator comm)
	{
		this(size, tag, comm.pid(), -1);
	}

	/** Retrieve a string representation of this object
	 * @return String
	 */
	public String toString ()
	{
		return size+" "+Remote.parseMType(tag)+" "+logID+"@"+pid;
	}

	/** Set payload size
	 * @param size Payload size
	 */
	public void setSize (long size)
	{
		this.size = size;
	}

	/** Retrieve payload size
	 * @return Payload size
	 */
	public long getSize ()
	{
		return size;
	}

	/** Retrieve the PeerID
	 * @return PeerID
	 */
	public PeerID pid ()
	{
		return pid;
	}

	/** Retrieve the message tag
	 * @return Message tag 
	 */
	public int tag ()
	{
		return tag;
	}

	/** Serial read function
	 * @param in Input stream 
	 */
	public void read (SerialIn in) throws IOException, InvocationTargetException,
		ClassNotFoundException, InstantiationException, IllegalAccessException
	{
		size = in.readLong();
		tag = in.readInt();
		pid = in.serialRead(PeerID.class);
		logID = in.readInt();	
	}
	
	/** Serial write function
	 * @param out Output stream
	 */
	public void write (SerialOut out) throws IOException, IllegalAccessException,
		InvocationTargetException, CTLException
	{
		out.writeLong(size);
		out.writeInt(tag);
		out.serialWrite(pid);
		out.writeInt(logID);
	}
}
