package CTL.Types;

import CTL.*;
import CTL.Serialize.*;
import CTL.Streams.*;
import ReflWrap.*;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

/** Implementation of ctl::reference in Java */
public class Reference<T extends Object> implements Writable, TemplHack
{	
	private static ObjectMap map = new ObjectMap();
	private boolean first = false;
	private long typeID = 0;
	private TypeTree type = null;

	public void setTypes (TypeTree[] types)
	{
		if (types!=null)
			type = types[0];
	}
	
	public boolean first ()
	{
		return first;
	}

	public boolean equals (Object moo)
	{
		if (!(moo instanceof Reference))
			return false;
		if (typeID==((Reference)moo).typeID())
			return true;
		return false;
	}

	public String toString()
	{
		return "Reference["+typeID+"] --> "+obj()+" ("+first+")";
	}

	public static void cleanup ()
	{
		map = new ObjectMap();
	}

	public static void debug ()
	{
		map.dump();
	}

	public long typeID ()
	{
		return typeID;
	}

	public T obj ()
	{
		return (T)map.getObj((int)typeID);
	}
	
	public static int unique ()
	{
		return map.getCounter();
	}
	
	public Reference (Object obj)
	{
		if (obj==null)
			return;
		typeID = obj.hashCode();
		if (map.getObj((int)typeID)==null)
		{
			map.regObj(obj);
			first = true;
		}
	}

	public void read (SerialIn in) throws IOException, ClassNotFoundException,
		InstantiationException, IllegalAccessException, InvocationTargetException
	{
		first = in.readBoolean();
		typeID = in.readLong();

		//System.out.println(typeID+" "+first+" "+type);

		if (first)
		{
			// TODO: Use the size
			in.readInt();
			T obj = (T)in.serialRead(type);
			if (obj==null)
				throw new IOException("Reference to a null Object.");
			map.regObj(obj, (int)typeID);
		}
	}

	public void write (SerialOut out) throws IOException, 
		IllegalAccessException, InvocationTargetException, CTLException
	{
		if (typeID==0)
		{
			out.writeBoolean(false);
			out.writeLong(-1);
			return;
		}

		out.writeBoolean(first);
		out.writeLong(typeID);

		//System.out.println(typeID+" "+first);

		if (!first)
			return;

		Object obj = map.getObj((int)typeID);
		out.writeInt(RUtil.objSize(obj));
		out.serialWrite(obj);
	}
}
