package CTL.Types;

import CTL.*;
import CTL.Serialize.*;
import CTL.Streams.*;
import ReflWrap.*;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

/** An element of the CTL tree */
public class Sibling<T extends Object> extends Tupel implements TemplHack
{	
	/** Template hack implementation
	 * @param types Type parameters
	 */
	public void setTypes (TypeTree[] types)
	{
		if (types != null)
		{
			type = new TypeTree[2];
			type[0] = types[0];
			try
			{
				type[1] = new TypeTree("java.util.LinkedList<CTL.Sibling<"+type[0]+">>");
			} 
			catch (ClassNotFoundException e) 
			{
				RUtil.except(e);
			}
		}
	}

	/** Generate a new Sibling
	 * @param data Data which this object should store
	 */
	public <T extends Object> Sibling (T data) throws CTLException,
		ClassNotFoundException
	{
		Class typ = (data!=null ? data.getClass() : Object.class);
		type = new TypeTree[2];
		type[0] = new TypeTree(typ);
		type[1] = new TypeTree("java.util.LinkedList<CTL.Sibling<"+typ.getName()+">>");
		this.data = new Object[type.length];

		setItem(0, data);
		setItem(1, new LinkedList<Sibling<T>>());
	}

	/** Search for a certain Sibling in the sub tree of this node
	 * @param data Data to search for
	 * @return Node which holds the data or null if none found
	 */
	public Sibling search (Object data) throws CTLException
	{
		if (item(0).equals(data))
			return this;
		
		for (Object moo : (LinkedList)item(1))
		{
			Sibling s = ((Sibling)moo).search(data);
			if (s!=null)
				return s;
		}
		
		return null;
	}

	/** Append a new Sibling to the subtree
	 * @param data Data to store
	 * @return Whether or not the new Sibling was succesfully appended
	 */
	public boolean append (Object data) throws CTLException,
		ClassNotFoundException
	{
		return append(new Sibling(data));
	}

	/** Append a new Sibling to the subtree
	 * @param sibl Sibling to append
	 * @return Whether or not the new Sibling was succesfully appended
	 */
	public boolean append (Sibling sibl) throws CTLException
	{
		return ((LinkedList)item(1)).add(sibl);
	}

	/** Retrieve the data associated to this Sibling
	 * @return Data
	 */
	public Object data () throws CTLException
	{
		return item(0);
	}

	/** Retrieve a String representation of this object
	 * @return String
	 */
	public String toString ()
	{
		try
		{
			StringBuffer buf = new StringBuffer();
			buf.append(item(0)+" ");
			if (item(1)!=null)
			{
				if (((LinkedList)item(1)).size()>0)
					buf.append("{ ");
				for (Object moo : (LinkedList)item(1))
					buf.append(moo);
				if (((LinkedList)item(1)).size()>0)
					buf.append("} ");
			}
			return buf.toString();
		}
		catch (CTLException e)
		{
			return "[ERR]";
		}
	}

	/** Serial read function
	 * @param in Input stream
	 */
	public void read (OIStream in) throws IOException, ClassNotFoundException, 
		InstantiationException, IllegalAccessException, InvocationTargetException
	{
		try
		{
			setItem(0, in.serialRead(type[0]));
			setItem(1, in.serialRead(type[1]));
		} 
		catch (CTLException e) 
		{
			RUtil.except(e);
		}	
	}
}
