package CTL.Types;

import CTL.Serialize.*;
import CTL.Streams.*;
import ReflWrap.*;

import java.io.*;
import java.lang.reflect.*;

/** CTL implementation of a simple tree */
public class Tree<T extends Object> implements Writable, TemplHack
{
	/** Root node of the tree */
	private Sibling root;
	/**	Type parameter for the data the siblings store */
	private TypeTree type = null;

	/** Template hack implementation
	 * @param types Type parameters
	 */
	public void setTypes (TypeTree[] types)
	{
		if (types!=null)
			type = types[0];
	}

	/** Generate a new Tree
	 * @param data Data associated to the root node
	 */
	public Tree (Object data) throws CTLException, ClassNotFoundException
	{
		root = new Sibling(data);
	}

	/** Retrieve this tree's root node 
	 * @return Root node
	 */
	public Sibling root ()
	{
		return root;
	}

	/** Search for a certain node
	 * @param data Data to search for
	 * @return Node which holds the data or null if none found
	 */
	public Sibling sibl (Object data) throws CTLException
	{
		return root.search(data);
	}

	/** Retrieve a string representation of this object
	 * @return String
	 */
	public String toString ()
	{
		return root.toString();
	}

	/** Check if this object is equal to another one
	 * @param moo Object to compare to
	 * @return True if equals, false if not
	 */
	public boolean equals (Object moo)
	{
		if (!(moo instanceof Tree))
			return false;
		return toString().equals(moo.toString());
	}
	
	/** Serial read function
	 * @param in Input stream
	 */
	public void read (SerialIn in) throws IOException, ClassNotFoundException, 
		InstantiationException, IllegalAccessException, InvocationTargetException
	{
		root = (Sibling)in.serialRead(type);
	}
	
	/** Serial write function
	 * @param out Output stream
	 */
	public void write (SerialOut out) throws IOException, IllegalAccessException,
		InvocationTargetException, CTLException
	{
		out.serialWrite(root);
	}
}
