package CTL;

import CTL.Serialize.*;
import CTL.Streams.*;
import ReflWrap.*;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

/** Result of a remote method call */
public class rResult
{
	/** Received status */
	private boolean[] recv;
	/** Return types */
	private TypeTree[] type;
	/** Return values */
	private Object[] value;

	/** Generate a new rResult storing one result
	 * @param type Return type
	 */
	public rResult (TypeTree type)
	{
		TypeTree[] args = new TypeTree[1];
		args[0] = type;
		create(args);
	}

	/** Generate a new rResult storing multiple results
	 * @param type Return types
	 */
	public rResult (TypeTree[] type)
	{
		create(type);
	}

	/** Underlying create() function for the constructors
	 * @param type List of types
	 */
	private void create (TypeTree[] type)
	{
		this.type = type;
		this.recv = new boolean[type.length];
		this.value = new Object[type.length];
		
		for (int i=0;i<type.length;i++)
		{
			recv[i] = false;
			value[i] = null;
		}
	}

	/** Retrieve the number of results stored
	 * @return Number of results
	 */
	public int results ()
	{
		return type.length;
	}

	/** Check whether a result was received
	 * @param i Index number
	 * @return True if a result was received, false otherwise
	 */
	public boolean received (int i)
	{
		return recv[i];
	}

	/** Retrieve the type of a return value
	 * @param i Index number
	 * @return Type of this return value
	 */
	public TypeTree type (int i)
	{
		return type[i];
	}

	/** Receive data for a specific return value
	 * @param in Input stream
	 * @param i Index number
	 */
	protected void recv (OIStream in, int i) throws IllegalAccessException, 
		IOException, InvocationTargetException, NoSuchMethodException,
		ClassNotFoundException
	{
		if (i>=type.length)
		{
			Env.log.log_msg(Logger.ERR, "Result "+i+" is out of bounds.\n");
			return;
		}

		Method temp = RUtil.serialrd2;
		Object[] args2 = new Object[1];
		args2[0] = (Object)type[i];
		
		Object args[] = new Object[2];
		args[0] = temp.invoke(in, args2);
		args[1] = i;

		Method foo = RUtil.getReceiver("Object");
		foo.invoke(this, args);
	}

	/** Function called by the remote side to pass the data
	 * @param val Return value
	 * @param i Index number
	 */
	private <T extends Object> void receive (T val, int i)
	{
		//System.out.println(this+" .:. "+i+" "+val);
		recv[i] = true;
		value[i] = val;
	}

	/** Return an integer value
	 * @param i Index number
	 * @return Value 
	 */
	public int intValue (int i)
	{
		return ((Integer)value[i]).intValue();
	}

	/** Return a double value
	 * @param i Index number
	 * @return Value
	 */
	public double doubleValue (int i)
	{
		return ((Double)value[i]).doubleValue();
	}

	/** Return a float value
	 * @param i Index number
	 * @return Value
	 */
	public float floatValue (int i)
	{
		return ((Float)value[i]).floatValue();
	}

	/** Return a String value
	 * @param i Index number
	 * @return Value
	 */
	public String StringValue (int i)
	{
		return (String)value[i];
	}

	/** Return an Object value
	 * @param i Index number
	 * @return Value
	 */
	public Object ObjectValue (int i)
	{
		return value[i];
	}

	/** Return a long value
	 * @param i Index number
	 * @return Value
	 */
	public long longValue (int i)
	{
		return ((Long)value[i]).longValue();
	}

	/** Return a short value
	 * @param i Index number
	 * @return Value
	 */
	public short shortValue (int i)
	{
		return ((Short)value[i]).shortValue();
	}

	/** Return a boolean value
	 * @param i Index value
	 * @return Value
	 */
	public boolean booleanValue (int i)
	{
		return ((Boolean)value[i]).booleanValue();
	}
}
