package CodeGen;

import java.io.*;
import java.util.*;

/** Abstract code generator interface */
public abstract class Gen
{
	/** Write a string to an output stream
	 * @param out OutputStreamWriter for the output stream
	 * @param str String to write
	 */
	protected static void writeTo (OutputStreamWriter out, String str) 
		throws IOException
	{
		out.write(str, 0, str.length());
	}

	/** Convert a class name to a Java source code filename
	 * @param klas Class name
	 * @return Filename
	 */
	protected static String classNameToDotJavaFile (String klas)
	{
		StringBuffer buf = new StringBuffer();
		StringTokenizer str = new StringTokenizer(klas, ".");

		while (str.hasMoreTokens())
			buf.append(str.nextToken()+((str.hasMoreTokens()) ? "/" : ""));
		buf.append(".java");
			
		return "src/"+buf.toString();
	}

	/** Retrieve the import statements from a Java source code file
	 * @param klas Class name
	 * @param buf Buffer to write to
	 */
	protected static void getImports (StringBuffer buf, String klas)
	{
		try
		{
			FileReader fin = new FileReader(classNameToDotJavaFile(klas));
			LineNumberReader lin = new LineNumberReader(fin);

			String line = lin.readLine();
			while (line!=null)
			{
				if ((line.length()>5) && (line.substring(0, 6).equals("import")))
					buf.append(line+"\n");
				line = lin.readLine();
			}

			fin.close();
		} catch (IOException e) {}
	}

	/** Use an external pretty printer
	 * @param file The File to modify
	 */
	protected static void prettyPrint (File file)
	{
		try
		{
			Runtime rt = Runtime.getRuntime();
			java.lang.Process proc = rt.exec("astyle "+file.toString());
			if (proc.waitFor()==0)
			{
				File orig = new File(file.toString()+".orig");
				orig.delete();
			}
		} catch (Exception e) {}
	}			
}
