package Hitman.HTTP;

import Hitman.*;
import java.io.*;
import java.net.*;
import java.util.*;

/** HTTP request */
public class Request extends NetRequest
{
	/** Possible request types */
	public enum Type { UNDEF, GET, POST };
	
	/** User agent */
	protected final static String uagent = "Hitman.HTTP v0.1";
	
	/** Header */
	public Header head = null;
	/** Possible payload */
	private byte[] payload = null;
	/** Server's reply */
	private byte[] reply = null;
	/** Request type */
	private Type type = Type.UNDEF;

	/** Constructor */
	public Request (HURL url)
	{
		this(url, Type.GET);
	}

	/** Constructor */
	public Request (HURL url, Type type)
	{
		this(url, type, (byte[])null, null);
	}

	/** POST request constructor */
	public Request (HURL url, Type type, String payload, String cntType)
	{
		this(url, type, payload.getBytes(), cntType);
	}

	/** POST request constructor */
	public Request (HURL url, Type type, byte[] payload, String cntType)	
	{
		super(url);
		this.type = type;
		this.payload = payload;

		head = new Header(TypeToString(type), url.path(), 
			(payload==null ? -1 : payload.length));
		if (payload!=null)
			head.addHeader("Content-Type", cntType);
	}

	/** Constructor from socket */
	public Request (Socket sock)
	{
		super(sock);
	}

	/** Retrieve reply 
	 * @return Data
	 */
	public byte[] getReply ()
	{
		return reply;
	}
	
	/** Get a new request
	 * @param in Input stream
	 */
	private void get_request (BufferedReader in) throws IOException
	{
		head = new Header(in);
		if (head.len>0)
			payload = IOUtil.slurp(in, head.len).getBytes();
	}

	/** Receive a request
	 * @return Output stream
	 */
	public Writer recv () throws IOException
	{
		if (sock==null)
			return null;
		get_request(getReader(sock));
		return getWriter(sock);
	}
	
	/** Send a request
	 * @param slurp Whether or not to read all data immediately
	 * @return Input stream
	 */
	public Reader send (boolean slurp) throws IOException
	{
		if (url==null)
			return null;
		//System.out.println(url.host()+" "+url.port());
		Socket sock = new Socket(url.host(), url.port());
		BufferedReader in = getReader(sock);
		BufferedWriter out = getWriter(sock);
		head.writeTo(out);
		if (payload!=null)
			out.write(payload.toString(), 0, payload.toString().length());
		out.flush();

		Header head = new Header(in);
		if (!slurp)
			return in;
		if (head.len>0)
		{
			reply = IOUtil.slurp(in, head.len).getBytes();
			if (reply.length!=head.len)
				System.err.println("Range not satisfied.");
		}
		else
			reply = IOUtil.slurp(in).getBytes();
		sock.close();
		return null;
	}

	/** Status report
	 * @return Status
	 */
	public NetStatus status ()
	{
		return null;
	}

	/** Helper function */
	private static String TypeToString (Type type)
	{
		switch (type)
		{
			case GET:
				return "GET";
			case POST:
				return "POST";
		}

		return "";
	}
}
