package Hitman;

import java.io.*;
import java.net.*;

/** Base class for requests */
public abstract class NetRequest
{
	/** URL */
	protected HURL url = null;
	/** Socket */
	protected Socket sock = null;

	/** Constructor from an URL */
	public NetRequest (HURL url)
	{
		this.url = url;
	}

	/** Constructor from a socket */
	public NetRequest (Socket sock)
	{
		this.sock = sock;
	}
	
	/** Get a BufferedReader for a certain socket
	 * @param sock Socket
	 * @return BufferedReader
	 */
	public static BufferedReader getReader (Socket sock)
		throws IOException
	{
		InputStream i = sock.getInputStream();
		InputStreamReader in = new InputStreamReader(i);
		return new BufferedReader(in);
	}

	/** Get a BufferedWriter for a certain socket
	 * @param sock Socket
	 * @return BufferedWriter
	 */
	public static BufferedWriter getWriter (Socket sock) 
		throws IOException
	{
		OutputStream o = sock.getOutputStream();
		OutputStreamWriter out = new OutputStreamWriter(o);
		return new BufferedWriter(out);
	}

	/** Receive, implementation classes need to implement this
	 * @return Writer
	 */
	public abstract Writer recv () throws IOException;
	
	/** Send, implementation classes need to implement this
	 * @param slurp If true, all data is read into an internal buffer,
	 * 	otherwise, it can be read via the returned Reader
	 * @return Reader
	 */
	public abstract Reader send (boolean slurp) throws IOException;

	/** Status report, implementation classes need to implement this
	 * @return Status
	 */
	public abstract NetStatus status ();
}
