package Test;

import CTL.*;
import CTL.Streams.*;
import CTL.Types.*;
import Impl.Types.*;

import java.io.*;
import java.lang.reflect.*;
import junit.framework.*;

/** Test case for the 'intelligent' stream
 * @see CTL.Streams.IStream
 */
public class TIStream extends TestCase
{
	private String[] tests = new String[4];
			
	public static String toString (Object data)
	{
		if (!data.getClass().isArray())
			return data.toString();

		StringBuffer ret = new StringBuffer();
		for (int i=0;i<java.lang.reflect.Array.getLength(data);i++)
			ret.append(java.lang.reflect.Array.get(data, i)+" ");
		return ret.toString();
	}

	public TIStream () throws IOException, ClassNotFoundException,
		InstantiationException, InvocationTargetException,
		IllegalAccessException, CTLException
	{
		PipedOutputStream pout = new PipedOutputStream();
		PipedInputStream pin = new PipedInputStream(pout);
		OOStream out = new OOStream(pout);
		OIStream in = new OIStream(pin);
		IStream intelli_stream = new IStream();
		
		Env.log = new Logger("test.log", Logger.DBG);
			
		int[] arr = new int[3];
		arr[0] = arr[1] = arr[2] = 3;
		double[] dbl = new double[2];
		dbl[0] = dbl[1] = 2.0;
		long[] lng = new long[2];
		lng[0] = lng[1] = 3;

		// Testcase 1
		intelli_stream.write((float)0.1);
		intelli_stream.write("foobar");
		intelli_stream.write(arr);
		intelli_stream.write(lng);
		intelli_stream.write(dbl);
		intelli_stream.write(1);
		intelli_stream.write((long)23);
		intelli_stream.write(true);
		intelli_stream.write(new DoubleDash(0.1, (float)0.1));
		intelli_stream.flush();
		tests[0] = intelli_stream.toString();

		// Testcase 2
		intelli_stream.writeToIStream(out);
		out.flush();
		IStream stream2 = new IStream();
		stream2.readFromStream(in);
		stream2.flush();
		tests[1] = stream2.toString();

		// Testcase 3
		intelli_stream.writeToStream(out, true);
		out.flush();
		StringBuffer buf = new StringBuffer();
		buf.append(in.readFloat()+"\n");
		buf.append(in.readString()+"\n");
		buf.append(toString(in.readArray(Integer.TYPE, null))+"\n");
		buf.append(toString(in.readArray(Long.TYPE, null))+"\n");
		buf.append(toString(in.readArray(Double.TYPE, null))+"\n");
		buf.append(in.readInt()+"\n");
		buf.append(in.readLong()+"\n");
		buf.append(in.readBoolean()+"\n");
		buf.append(in.serialRead(DoubleDash.class));
		tests[2] = buf.toString();
		
		// Testcase 4
		tests[3] = intelli_stream.toString();
	}

	public void testCase1 ()
	{
		for (int i=0;i<tests.length;i++)
			assertTrue(tests[i].equals(tests[0]));
	}
	
	public void testCase2 ()
	{
		for (int i=0;i<tests.length;i++)
			assertTrue(tests[i].equals(tests[1]));
	}
	
	public void testCase3 ()
	{
		for (int i=0;i<tests.length;i++)
			assertTrue(tests[i].equals(tests[2]));
	}
	
	public void testCase4 ()
	{
		for (int i=0;i<tests.length;i++)
			assertTrue(tests[i].equals(tests[3]));
	}

	public static void main (String[] args)
	{
		try
		{
			TIStream mango = new TIStream();
			mango.testCase1();
			mango.testCase2();
			mango.testCase3();
			mango.testCase4();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
}
