package Tools;

/** Simple byte array to hex conversion class */
public class HexDumper
{
	/** Byte array to work with */
	private byte[] bytes;
	
	/** Generate a new HexDumper
	 * @param bytes Byte array to work with
	 */
	public HexDumper (byte[] bytes)
	{
		this.bytes = bytes;
	}

	/** Generate a new HexDumper
	 * @param str String to work with
	 */
	public HexDumper (String str)
	{
		this.bytes = str.getBytes();
	}

	/** Get a string representation of this object
	 * @return String
	 */
	public String toString ()
	{
		return convert();
	}

	/** Convert to a string of hexadecimal numbers
	 * @return String
	 */
	public String convert ()
	{
		StringBuffer buf = new StringBuffer();
		int i;
		for (i=0;i<bytes.length;i++)
		{
			if (i%16==0)
				buf.append(lineNumber((i/16)+1, 8)+" ");
			buf.append(byteToHex(bytes[i]));
			bytes[i]=cleanByte(bytes[i]);
			if ((i+1)%16==0)
				buf.append("  "+new String(bytes, i-15, 16)+"\n");
			else
				buf.append(" ");
		}
		for (int j=0;j<((16*3+1)-((i%16)*3));j++)
			buf.append(" ");
		buf.append(new String(bytes, bytes.length-(i%16), i%16));
		return buf.toString();
	}

	/** Filter escape characters
	 * @param b A byte
	 * @return Filtered byte
	 */
	public static byte cleanByte (byte b)
	{
		if ((b>6)&&(b<14))
			return 46;
		return b;
	}

	/** Pretty line number
	 * @param i Line number
	 * @param w Width of the line number column
	 * @return Correctly aligned line number in a string
	 */
	public static String lineNumber (int i, int w)
	{
		int a=i;
		int b=0;

		while (a!=0)
		{
			b++;
			a/=2;
		}

		byte[] ln = new byte[b--];

		String align = "";
		if (b<w)
		{
			StringBuffer buf = new StringBuffer();
			for (int j=1;j<w-b;j++)
				buf.append("0");
			align = buf.toString();
		}
		
		while(i!=0)
		{
			ln[b--]=(new Integer((i%2)+48)).byteValue();
			i/=2;
		}	

		return align+(new String(ln));
	}

	/** Convert one byte to hexadecimal
	 * @param beit A byte
	 * @return Hexadecimal string
	 */
	public static String byteToHex (byte beit)
	{
		int[] hex = new int[2];
		hex[0] = beit & 0x0F;
		hex[1] = (beit & 0xF0) / 16;
		return intToHex(hex[1])+""+intToHex(hex[0]);
	}

	/** Convert one integer to hexadecimal
	 * @param hbyte An integer
	 * @return Hexadecimal string
	 */
	public static char intToHex (int hbyte)
	{
		if (hbyte<10)
			return (char)(hbyte+48);
		return (char)(hbyte+55);
	}
	
	public static void main (String[] args)
	{
		HexDumper dump = new HexDumper("AAAAAAAAAAAAAAAAFGH \n");
		System.out.println(dump.convert());
	}
}
