package CTL.Serialize;

import CTL.*;
import CTL.Types.CTLException;
import ReflWrap.*;

import java.io.*;
import java.lang.reflect.*;

/** Human-readable output.
 * 	Acts as a proof-of-concept for pluggable serializers.
 */
public class ASCIIOut extends CTLOut
{
	private CTLOut file;
	
	public ASCIIOut (OutputStream out, OutputStream file) throws IOException
	{
		super(out);
		this.file = new CTLOut(file);
	}

	private void dump (String s) throws IOException
	{
		file.writeBytes(s+"\n");
	}

	public void writeBoolean (boolean data) throws IOException
	{
		super.writeBoolean(data);
		dump("Boolean: "+data);
	}
		
	public void writeByte (byte data) throws IOException
	{
		super.writeByte(data);
		dump("Byte: "+data);
	}
	
	public void writeChar (char data) throws IOException
	{
		super.writeChar(data);
		dump("Char: "+data);
	}
	
	public void writeDouble (double data) throws IOException
	{
		super.writeDouble(data);
		dump("Double: "+data);
	}
		
	public void writeFloat (float data) throws IOException
	{
		super.writeFloat(data);
		dump("Float: "+data);
	}
		
	public void writeInt (int data) throws IOException
	{
		super.writeInt(data);
		dump("Int: "+data);
	}
		
	public void writeLong (long data) throws IOException
	{
		super.writeLong(data);
		dump("Long: "+data);
	}
		
	public void writeShort (short data) throws IOException
	{
		super.writeShort(data);
		dump("Short: "+data);
	}
	
	public void writeString (String data) throws IOException
	{
		super.writeString(data);
		dump("String: "+data);
	}
		
	public void writeWString (String data) throws IOException
	{
		super.writeWString(data);
		dump("WString: "+data);
	}
	
	protected void writeData (Object data) throws IOException, 
		IllegalAccessException, InvocationTargetException, CTLException
	{
		super.writeData(data);
		((Writable)data).write(file);
	}
	
	public <T extends Object> void writeArray (T data) throws 
		IOException, IllegalAccessException, InvocationTargetException
	{
		super.writeArray(data);
		
		if (data==null)
		{
			writeInt(0);
			return;
		}

		writeInt(java.lang.reflect.Array.getLength(data));
		__writeArray(data);
	}
		
	public <T extends Object> void __writeArray (T data) throws 
		IOException, IllegalAccessException, InvocationTargetException
	{
		super.__writeArray(data);
		
		if (data==null)
			return;

		String type = null;

		try
		{
			type = java.lang.reflect.Array.get(data, 0).getClass().getName();
		}
		catch (NullPointerException e)
		{
			type = Refl.arrayType(data.getClass().getName());
		}

		Method m = RUtil.getWriter(type);
		Object[] args = new Object[1];
		
		for (int i=0;i<java.lang.reflect.Array.getLength(data);i++)
		{
			args[0] = java.lang.reflect.Array.get(data, i);
			m.invoke(this, args);
		}
	}
	
	public void writeObject (Object data) throws IOException, 
		IllegalAccessException, InvocationTargetException, CTLException
	{
		super.writeObject(data);
		file.writeObject(data);
	}

	public void flush () throws IOException
	{
		super.flush();
		file.flush();
	}

	public void close () throws IOException
	{
		super.close();
		file.close();
	}
}
